import { CytoscapeLayout, CytoscapeProvider } from './cytoscape-layouts';
import { GraphologyLayout, GraphologyProvider } from './graphology-layouts';

export type GraphologyLayoutAlgorithms =
  | `Graphology_circular`
  | `Graphology_random`
  | `Graphology_noverlap`
  | `Graphology_forceAtlas2`
  | `Graphology_forceAtlas2_webworker`;

export type CytoscapeLayoutAlgorithms =
  | 'Cytoscape_klay'
  | 'Cytoscape_dagre'
  | 'Cytoscape_elk'
  | 'Cytoscape_fcose'
  | 'Cytoscape_cose-bilkent'
  | 'Cytoscape_cise'
  | 'Cytoscape_cose'
  | 'Cytoscape_grid'
  | 'Cytoscape_circle'
  | 'Cytoscape_concentric'
  | 'Cytoscape_breadthfirst';

export type AllLayoutAlgorithms = GraphologyLayoutAlgorithms | CytoscapeLayoutAlgorithms;

export type Providers = GraphologyProvider | CytoscapeProvider;
export type LayoutAlgorithm<Provider extends Providers> = `${Provider}_${string}`;

export type AlgorithmToLayoutProvider<Algorithm extends AllLayoutAlgorithms> = Algorithm extends GraphologyLayoutAlgorithms
  ? GraphologyLayout
  : Algorithm extends CytoscapeLayoutAlgorithms
  ? CytoscapeLayout
  : CytoscapeLayout | GraphologyLayout;

export enum Layouts {
  KLAY = 'Cytoscape_klay',
  DAGRE = 'Cytoscape_dagre',
  ELK = 'Cytoscape_elk',
  FCOSE = 'Cytoscape_fcose',
  COSE_BILKENT = 'Cytoscape_cose-bilkent',
  CISE = 'Cytoscape_cise',
  GRID = 'Cytoscape_grid',
  COSE = 'Cytoscape_cose',
  CIRCLE = 'Cytoscape_circle',
  CONCENTRIC = 'Cytoscape_concentric',
  BREATHFIRST = 'Cytoscape_breadthfirst',
  RANDOM = 'Graphology_random',
  CIRCULAR = 'Graphology_circular',
  NOVERLAP = 'Graphology_noverlap',
  FORCEATLAS2 = 'Graphology_forceAtlas2',
  FORCEATLAS2WEBWORKER = 'Graphology_forceAtlas2_webworker',
}
