// import { handles } from '@graphpolaris/shared/lib/querybuilder/usecases';
import { useQuerybuilderGraph } from '@graphpolaris/shared/lib/data-access';
import React, { useMemo, useState } from 'react';
import { Position } from 'reactflow';
import { NodeAttribute, SchemaReactflowEntityNode } from '../../../model';
import { PillDropdown } from '../../pilldropdown/pilldropdown';
import { FilterHandle } from '../../FilterHandle';

/**
 * Component to render an entity flow element
 * @param {NodeProps} param0 The data of an entity flow element.
 */
export const EntityFlowElement = React.memo((node: SchemaReactflowEntityNode) => {
  const data = node.data;
  if (!data.leftRelationHandleId) throw new Error('EntityFlowElement: data.leftRelationHandleId is undefined');
  if (!data.rightRelationHandleId) throw new Error('EntityFlowElement: data.rightRelationHandleId is undefined');

  const graph = useQuerybuilderGraph();
  const attributeEdges = useMemo(
    () => graph.edges.filter((edge) => edge.source === node.id && !!edge?.attributes?.sourceHandleData.attributeType),
    [graph],
  );

  const [hovered, setHovered] = useState(false);
  const [handleBeingDragged, setHandleBeingDragged] = useState(-1);

  const onMouseEnter = (event: React.MouseEvent) => {
    if (!hovered) setHovered(true);
  };

  const onMouseLeave = (event: React.MouseEvent) => {
    if (hovered) setHovered(false);
  };

  const onHandleMouseDown = (attribute: NodeAttribute, i: number, event: React.MouseEvent) => {
    setHandleBeingDragged(i);
    window.addEventListener('mouseup', onHandleMouseUp, true);
  };

  const onHandleMouseUp = () => {
    setHandleBeingDragged(-1);
    window.removeEventListener('mouseup', onHandleMouseUp, true);
  };

  const onConnect = (params: any) => {
    console.log('EntityPill onConnect', params);
  };

  return (
    <div className="p-3 bg-transparent" onMouseEnter={onMouseEnter} onMouseLeave={onMouseLeave}>
      <div className={`rounded-sm shadow min-w-[9rem] max-w-[9rem] text-[0.8rem] bg-gradient-to-r pt-1 from-[#FFA952] to-[#D66700]`}>
        <div className={`pt-1 ${data.selected ? 'bg-secondary-400' : 'bg-secondary-50'}`}>
          <FilterHandle
            handle={data.leftRelationHandleId}
            type="target"
            position={Position.Left}
            // className={'!top-8 !left-2 !bg-danger-700 !rounded-none'}
            // outerClassName={'!bg-blue-700 !rounded-none'}
            className={'!bg-accent-700'}
          />
          <FilterHandle
            handle={data.rightRelationHandleId}
            type="source"
            position={Position.Right}
            // outerClassName={'!bg-blue-700 !rounded-none'}
            className={'!bg-accent-700'}
          />
          <div className="">
            <div className="text-center py-1">{data.name}</div>
          </div>
          {data?.attributes && (
            <PillDropdown
              node={node}
              attributes={data.attributes}
              attributeEdges={attributeEdges.map((edge) => edge?.attributes)}
              hovered={hovered}
              handleBeingDraggedIdx={handleBeingDragged}
              onHandleMouseDown={onHandleMouseDown}
            />
          )}
        </div>
      </div>
    </div>
  );
});

EntityFlowElement.displayName = 'EntityFlowElement';

export default EntityFlowElement;
