import { categoricalColors } from 'config';
import { MultiGraph } from 'graphology';
import { AugmentedEdgeAttributes, GraphData, VisualRegionConfig } from './components/types';
import { GraphQueryResult, Node } from '@graphpolaris/shared/lib/data-access/store/graphQueryResultSlice';

const buildGraphology = (data: GraphData): MultiGraph => {
  const graph = new MultiGraph();

  const nodeMap = new Map<string, Node>();
  data.nodes.forEach((node) => {
    nodeMap.set(node.id, node);
  });

  const nodeEntries = data.nodes.map((node) => ({
    key: node.id,
    attributes: {
      ...node.attributes,
      label: node.label,
    },
  }));

  graph.import({ nodes: nodeEntries });

  const validEdgeEntries = data.edges
    .filter((edge) => nodeMap.has(edge.from) && nodeMap.has(edge.to))
    .map((edge) => ({
      source: edge.from,
      target: edge.to,
    }));

  graph.import({ edges: validEdgeEntries });

  return graph;
};

const numColorsCategorical = Object.keys(categoricalColors.lightMode).length;
export const isColorCircleFix = true;
let config: any = {};
if (isColorCircleFix) {
  config = {
    circle: {
      fillClr: 'hsl(var(--clr-sec--500))',
      strokeClr: 'hsl(var(--clr-sec--500))',
    },
    edges: {
      stroke: 'bg-secondary-600',
      strokeWidth: '2',
      strokeOpacity: '0.7',
    },

    brush: {
      fillClr: 'hsl(var(--clr-sec--100))',
      strokeClr: 'black',
    },
  };
} else {
  config = {
    circle: {
      fillClr: 'NO_USED',
      strokeClr: 'hsl(var(--clr-sec--500))',
    },
    edges: {
      stroke: 'NO_USED',
      strokeWidth: '2',
      strokeOpacity: '0.7',
    },
    brush: {
      fillClr: 'hsl(var(--clr-sec--100))',
      strokeClr: 'black',
    },
  };
}

const marginAxis = 0.2;

const configVisualRegion: VisualRegionConfig = {
  marginPercentage: { top: 0.14, right: 0.15, bottom: 0.2, left: 0.15 },
  margin: { top: 0.0, right: 0.0, bottom: 0.0, left: 0.0 },
  width: 600,
  widthPercentage: 0.4,
  height: 300,
  widthMargin: 0.0,
  heightMargin: 0.0,
};

configVisualRegion.margin = {
  top: configVisualRegion.marginPercentage.top * configVisualRegion.height,
  right: configVisualRegion.marginPercentage.right * configVisualRegion.width,
  bottom: configVisualRegion.marginPercentage.bottom * configVisualRegion.height,
  left: configVisualRegion.marginPercentage.left * configVisualRegion.width,
};

configVisualRegion.widthMargin = configVisualRegion.width - configVisualRegion.margin.right - configVisualRegion.margin.left;
configVisualRegion.heightMargin = configVisualRegion.height - configVisualRegion.margin.top - configVisualRegion.margin.bottom;

export { buildGraphology, numColorsCategorical, config, configVisualRegion, marginAxis };
